<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

class Zakeke_AJAX {

	/**
	 * Hook in ajax handlers.
	 */
	public static function init() {
        add_action( 'wc_ajax_zakeke_get_auth', array( __CLASS__, 'get_auth' ) );
		add_action( 'wc_ajax_zakeke_get_price', array( __CLASS__, 'get_price' ) );
        add_action( 'wc_ajax_zakeke_get_configurator_price', array( __CLASS__, 'get_configurator_price' ) );
        add_action( 'wc_ajax_zakeke_share', array( __CLASS__, 'share' ) );
        add_action( 'wc_ajax_zakeke_update_cart', array( __CLASS__, 'update_cart' ) );
	}

    /**
     * Get a client-to-server authentication token.
     */
    public static function get_auth() {
        ob_start();

        $auth = zakeke_get_auth();

        $user_id = get_current_user_id();
        if ( $user_id > 0 ) {
            $auth->set_customer( (string) $user_id );
        } else {
            $auth->set_guest( zakeke_guest_code() );
        }

        try {
            wp_send_json( array(
                'token'      => $auth->get_auth_token()
            ) );
        } catch (Exception $e) {
            wp_send_json( array(
                'error' => $e->getMessage()
            ) );
        }

        die();
    }

	/**
	 * Get a matching variation price based on posted attributes.
	 */
	public static function get_price() {
		ob_start();

		if ( ! empty( $_POST['product_id'] ) ) {
			$product_id = $_POST['product_id'];
		} else {
			$product_id = $_POST['add-to-cart'];
		}

        $qty = 1;
		if ( isset( $_POST['quantity'] ) ) {
            // Sanitize
            $qty = wc_stock_amount(preg_replace("/[^0-9\.]/", '', $_POST['quantity']));
            if ($qty <= 0) {
                $qty = 1;
            }
        }

		if ( ! ( $product = wc_get_product( absint( $product_id ) ) ) ) {
			die();
		}

		if ( $product->is_type( 'variable' ) ) {
			/** @var WC_Product_Data_Store_Interface $data_store */
            $data_store = WC_Data_Store::load( 'product' );
            $variation_id = $data_store->find_matching_product_variation( $product, wp_unslash( $_POST ) );
			if ( $variation_id ) {
			    $product = wc_get_product( $variation_id );
            }
		}

        do_action( 'zakeke_before_ajax_price', $product, $qty );

        $integration = new Zakeke_Integration();
        $hide_price = $integration->hide_price;

        $original_price = 0.0;
        $zakeke_final_price = 0.0;

		if ($hide_price !== 'yes') {
            $zakeke_price = 0.0;
		    $original_price = (float) wc_get_price_to_display( $product, array( 'qty' => $qty ) );

            if (isset($_POST['zakeke-percent-price']) && $_POST['zakeke-percent-price'] > 0.0) {
                $zakeke_price += $original_price * ((float)$_POST['zakeke-percent-price'] / 100);
            }
            if (isset($_POST['zakeke-price']) && $_POST['zakeke-price'] > 0.0) {
                $zakeke_price += (float)$_POST['zakeke-price'];
            }

            $zakeke_final_price = (float) wc_get_price_to_display( $product, array( 'price' => $zakeke_price ) );
        }

		wp_send_json( array(
            'is_purchasable'      => $product->is_purchasable(),
            'is_in_stock'         => $product->is_in_stock(),
            'price_including_tax' => $original_price + $zakeke_final_price
        ) );

		die();
	}

    /**
     * Get a matching variation price based on posted attributes.
     */
    public static function get_configurator_price() {
        ob_start();

        if ( ! empty( $_POST['product_id'] ) ) {
            $product_id = $_POST['product_id'];
        } else {
            $product_id = $_POST['add-to-cart'];
        }

        // Sanitize
        $qty = wc_stock_amount( preg_replace( '/[^0-9\.]/', '',  $_POST['quantity'] ) );
        if ($qty <= 0) {
            $qty = 1;
        }

        if ( ! ( $product = wc_get_product( absint( $product_id ) ) ) ) {
            die();
        }

        if ( $product->is_type( 'variable' ) ) {
            /** @var WC_Product_Data_Store_Interface $data_store */
            $data_store = WC_Data_Store::load( 'product' );
            $variation_id = $data_store->find_matching_product_variation( $product, wp_unslash( $_POST ) );
            if ( $variation_id ) {
                $product = wc_get_product( $variation_id );
            }
        }

        do_action( 'zakeke_before_ajax_configurator_price', $product, $qty );

        $integration = new Zakeke_Integration();
        $hide_price = $integration->hide_price;

        $original_price = 0.0;
        $zakeke_price = 0.0;

        if ($hide_price !== 'yes') {
            $original_price = (float) wc_get_price_to_display( $product );

            if (isset($_POST['zakeke_price']) && $_POST['zakeke_price'] > 0.0) {
                $zakeke_price += (float)$_POST['zakeke_price'];
            }

            $zakeke_final_price = (float) wc_get_price_to_display( $product, array( 'price' => $zakeke_price ) );
        }

        wp_send_json( array(
            'is_purchasable'      => $product->is_purchasable(),
            'is_in_stock'         => $product->is_in_stock(),
            'price_including_tax' => $original_price + $zakeke_final_price
        ) );

        die();
    }

    public static function share() {
        ob_start();

        if (!isset($_REQUEST['path'])) {
            wp_send_json(array(
                'error' => __('Missing path parameter')
            ));

            die();
        }

        $path = $_REQUEST['path'];
        $response = zakeke_retry(function () use ($path) {
            $response = wp_remote_request( ZAKEKE_BASE_URL . '/SharedPreview' . $path );
            if ( is_wp_error( $response )
                || ( is_array( $response )
                    && $response['response']['code']
                    && floor( $response['response']['code'] ) / 100 >= 4 )
            ) {
                throw new Exception( 'Zakeke_AJAX::share ' . $path);
            }

            return $response;
        });

        header('Content-type: image/png');
        echo wp_remote_retrieve_body( $response );
        die();
    }

    public static function update_cart()
    {
        ob_start();

        $webservice = new Zakeke_Webservice();
        $integration = new Zakeke_Integration();

        foreach (WC()->cart->get_cart() as $cart_item_key => $values) {
            if (!isset($values['zakeke_data'])) {
                continue;
            }

            $zakeke_data = $values['zakeke_data'];

            $cart_item_data = &WC()->cart->cart_contents[$cart_item_key];

            $qty = $cart_item_data['quantity'];
            if ($qty <= 0) {
                $qty = 1;
            }

            $zakeke_cart_data = $webservice->cart_info($zakeke_data['design'], $qty);

            $cart_item_data['zakeke_data']['previews'] = $zakeke_cart_data->previews;

            if ($integration->hide_price !== 'yes') {
                $cart_item_data['zakeke_data']['pricing'] = $zakeke_cart_data->pricing;

                $original_price = $zakeke_data['original_final_price'];

                $zakeke_price = zakeke_calculate_price(
                    $original_price,
                    $cart_item_data['zakeke_data']['pricing'],
                    $qty
                );

                /** @var WC_Product $product */
                $product = $values['data'];

                if (get_option('woocommerce_tax_display_shop') === 'excl') {
                    $zakeke_tax_price = (float)wc_get_price_excluding_tax($product, array('price' => $zakeke_price));
                } else {
                    $zakeke_tax_price = (float)wc_get_price_including_tax($product, array('price' => $zakeke_price));
                }

                $zakeke_excl_tax_price = (float)wc_get_price_excluding_tax($product, array('price' => $zakeke_price));

                $cart_item_data['zakeke_data']['price'] = $zakeke_price;
                $cart_item_data['zakeke_data']['price_tax'] = $zakeke_tax_price;
                $cart_item_data['zakeke_data']['price_excl_tax'] = $zakeke_excl_tax_price;

                $product->set_price($original_price + $zakeke_price);
            }

            WC()->cart->set_session();
        }

        wp_send_json(array(
            'return_url' => wc_get_cart_url()
        ));

        die();
    }
}

Zakeke_AJAX::init();
